@@ -1,6 +1,8 @@
 """Utils to get right Date parsing function."""
+import datetime
 
 from dateutil import parser
+from pytz import UTC
 
 date_helper = None
 
@@ -8,6 +10,15 @@ date_helper = None
 class DateHelper:
     """DateHelper to groups different implementations of date operations."""
 
+    def __init__(self, timezone: datetime.tzinfo = UTC) -> None:
+        """
+        Initialize defaults.
+
+        :param timezone: Default timezone used for serialization "datetime" without "tzinfo".
+                         Default value is "UTC".
+        """
+        self.timezone = timezone
+
     def parse_date(self, date_string: str):
         """
         Parse string into Date or Timestamp.
@@ -30,6 +41,18 @@ class DateHelper:
 
         return nanoseconds_in_days + nanoseconds_in_seconds + nanoseconds_in_micros
 
+    def to_utc(self, value: datetime):
+        """
+        Convert datetime to UTC timezone.
+
+        :param value: datetime
+        :return: datetime in UTC
+        """
+        if not value.tzinfo:
+            return self.to_utc(value.replace(tzinfo=self.timezone))
+        else:
+            return value.astimezone(UTC)
+
 
 def get_date_helper() -> DateHelper:
     """
